@@ -16,7 +16,8 @@ import re
 import htmlentitydefs
 
 NS = {'ds': 'http://www.w3.org/2000/09/xmldsig#'}
-DS = ElementMaker(namespace=NS['ds'], nsmap=NS)
+NSDefault = {None: 'http://www.w3.org/2000/09/xmldsig#'}
+DS = ElementMaker(namespace=NS['ds'], nsmap=NSDefault)
 
 # Enable this to get various parts written to files in /tmp. Not for production!
 _DEBUG_WRITE_TO_FILES = False
@@ -33,7 +34,7 @@ ASN1_BER_ALG_DESIGNATOR_PREFIX = {
 
 TRANSFORM_ENVELOPED_SIGNATURE = 'http://www.w3.org/2000/09/xmldsig#enveloped-signature'
 TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS = 'http://www.w3.org/2001/10/xml-exc-c14n#WithComments'
-TRANSFORM_C14N_EXCLUSIVE = 'http://www.w3.org/2001/10/xml-exc-c14n'
+TRANSFORM_C14N_EXCLUSIVE = 'http://www.w3.org/2001/10/xml-exc-c14n#'
 TRANSFORM_C14N_INCLUSIVE = 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315'
 
 ALGORITHM_DIGEST_SHA1 = "http://www.w3.org/2000/09/xmldsig#sha1"
@@ -106,7 +107,7 @@ def _load_keyspec(keyspec, private=False, signature_element=None):
     Possible keyspecs, in evaluation order :
 
       - a callable.    Return a partial dict with 'f_private' set to the keyspec.
-      - a filename.    Load an X.509 certificate from the file.
+      - a filename.    Load a PEM X.509 certificate from the file.
       - a PKCS#11-URI  (see xmlsec.pk11.parse_uri()). Return a dict with 'f_private'
                        set to a function calling the 'sign' function for the key,
                        and the rest based on the (public) key returned by
@@ -168,13 +169,14 @@ def _load_keyspec(keyspec, private=False, signature_element=None):
 
     res = {'keyspec': keyspec,
            'source': source,
-           'data': data,
            'key': key,
            'keysize': int(key.size()) + 1}
 
     if private:
         res['f_private'] = key_f_private or rsa_x509_pem.f_private(key)
+        res['data'] = data  # TODO - normalize private keyspec too!
     else:
+        res['data'] = cert_pem['pem']  # normalized PEM
         res['f_public'] = rsa_x509_pem.f_public(key)
 
     return res
@@ -270,7 +272,7 @@ def _alg(elt):
     if uri is None:
         return None
     else:
-        return uri.rstrip('#')
+        return uri
 
 
 def _remove_child_comments(t):
@@ -311,6 +313,9 @@ def _process_references(t, sig=None, return_verified=True):
             logging.debug("transform: %s" % _alg(tr))
             obj = _transform(_alg(tr), obj, tr)
 
+        if not isinstance(obj, basestring):
+            obj = _unescape(etree.tostring(obj).decode("utf8", 'replace')).encode("utf8").strip()
+
         if _DEBUG_WRITE_TO_FILES:
             with open("/tmp/foo-obj.xml", "w") as fd:
                 fd.write(obj)
@@ -396,7 +401,6 @@ def _enveloped_signature(t):
             fd.write(etree.tostring(t))
     return t
 
-
 def _c14n(t, exclusive, with_comments, inclusive_prefix_list=None):
     """
     Perform XML canonicalization (c14n) on an lxml.etree.
@@ -523,7 +527,8 @@ def _verify(t, keyspec):
 
         hash_alg, ref = _process_references(t, sig)
         si = sig.find(".//{%s}SignedInfo" % NS['ds'])
-        b_digest = _create_signature_digest(si, hash_alg)
+        cm_alg = _cm_alg(si)
+        b_digest = _create_signature_digest(si, cm_alg, hash_alg)
 
         actual = _signed_value(b_digest, this_keysize, True, hash_alg)
         expected = this_f_public(b64d(sv))
@@ -565,11 +570,18 @@ def _enveloped_signature_template(c14n_method, digest_alg, transforms, reference
     )
 
 
-def add_enveloped_signature(t, c14n_method=TRANSFORM_C14N_INCLUSIVE, digest_alg=ALGORITHM_DIGEST_SHA1, transforms=None,
-                            reference_uri=""):
+def add_enveloped_signature(t,
+                            c14n_method=TRANSFORM_C14N_INCLUSIVE,
+                            digest_alg=ALGORITHM_DIGEST_SHA1,
+                            transforms=None,
+                            reference_uri='',
+                            pos=0):
     if transforms is None:
         transforms = (TRANSFORM_ENVELOPED_SIGNATURE, TRANSFORM_C14N_EXCLUSIVE_WITH_COMMENTS)
-    _root(t).insert(0, _enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri))
+    if pos == -1:
+        _root(t).append(_enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri))
+    else:
+        _root(t).insert(pos, _enveloped_signature_template(c14n_method, digest_alg, transforms, reference_uri))
 
 
 def sign(t, key_spec, cert_spec=None, reference_uri=''):
@@ -597,11 +609,10 @@ def sign(t, key_spec, cert_spec=None, reference_uri=''):
 
     public = _load_keyspec(cert_spec)
     if public is None:
-        raise XMLSigException("Unable to load public key from '%s'" \
-                              % (cert_spec))
+        raise XMLSigException("Unable to load public key from '%s'" % cert_spec)
     if public['keysize'] != private['keysize']:
-        raise XMLSigException("Public and private key sizes do not match (%s, %s)" \
-                                  % (public['keysize'], private['keysize']))
+        raise XMLSigException("Public and private key sizes do not match (%s, %s)"
+                              % (public['keysize'], private['keysize']))
     logging.debug("Using %s bit key" % (private['keysize']))
 
     if t.find(".//{%s}Signature" % NS['ds']) is None:
@@ -613,10 +624,12 @@ def sign(t, key_spec, cert_spec=None, reference_uri=''):
 
     for sig in t.findall(".//{%s}Signature" % NS['ds']):
         hash_alg = _process_references(t, sig, return_verified=False)
+        # XXX create signature reference duplicates/overlaps process references unless a c14 is part of transforms
         si = sig.find(".//{%s}SignedInfo" % NS['ds'])
-        b_digest = _create_signature_digest(si, hash_alg)
+        cm_alg = _cm_alg(si)
+        b_digest = _create_signature_digest(si, cm_alg, hash_alg)
 
-        # RSA sign hash digest and insert it into the XML
+        # sign hash digest and insert it into the XML
         tbs = _signed_value(b_digest, private['keysize'], do_padding, hash_alg)
         signed = private['f_private'](tbs)
         signature = b64e(signed)
@@ -631,14 +644,18 @@ def sign(t, key_spec, cert_spec=None, reference_uri=''):
     return t
 
 
-def _create_signature_digest(si, hash_alg):
-    """
-    :param hash_alg: string such as 'sha1'
-    """
+def _cm_alg(si):
     cm = si.find(".//{%s}CanonicalizationMethod" % NS['ds'])
     cm_alg = _alg(cm)
     if cm is None or cm_alg is None:
         raise XMLSigException("No CanonicalizationMethod")
+    return cm_alg
+
+
+def _create_signature_digest(si, cm_alg, hash_alg):
+    """
+    :param hash_alg: string such as 'sha1'
+    """
     sic = _transform(cm_alg, si)
     logging.debug("SignedInfo C14N: %s" % sic)
     digest = _digest(sic, hash_alg)
